@@ -354,6 +354,8 @@ class RequestStoreField(StoreField):
         return self.format_value(ar,v)
         
     def format_value(self,ar,v):
+        if v is None:
+            raise Exception("Got None value for %s" % self)
         n = v.get_total_count()
         if n == 0:
             return ''
@@ -757,7 +759,8 @@ class Store:
         dtl = rh.report.get_detail()
         if dtl:
             dh = dtl.get_handle(rh.ui)
-            self.collect_fields(self.detail_fields,*dh.lh_list)
+            #~ self.collect_fields(self.detail_fields,*dh.lh_list)
+            self.collect_fields(self.detail_fields,dh)
         
         
         if issubclass(rh.report,table.Table):
@@ -1035,7 +1038,7 @@ class Store:
             
     def parse_params(self,request,**kw):
         pv = request.REQUEST.getlist(ext_requests.URL_PARAM_PARAM_VALUES)
-        
+        #~ logger.info("20120221 parse_params(%s)",pv)
         def parse(self,form_value):
             if form_value == '' and not self.field.empty_strings_allowed:
                 return self.form2obj_default
@@ -1048,11 +1051,15 @@ class Store:
             else:
                 return self.parse_form_value(form_value,None)
       
-        if pv: 
+        #~ if pv: 
+            #~ if len(self.param_fields) != len(pv):
+                #~ raise Exception("len(%r) != len(%r)" % (self.param_fields,pv))
+        if pv and len(self.param_fields) == len(pv):
             for i,f in enumerate(self.param_fields):
                 kw[f.field.name] = parse(f,pv[i])
         else:
             for i,f in enumerate(self.param_fields):
                 kw[f.field.name] = parse(f,'')
+        #~ logger.info("20120221 2 parse_params(%s) --> %r",pv,kw)
         return kw
         
